/*
 * Project name: Consolidated Library for MQ Sensors
 * Copyright:
      (c) mikroElektronika, 2014 - Sourcecode
 *Copyright Notice:
      (c) HavenTechnik, 2019 - Consolidation into single library for various devices
 * Revision History:
      20140718:
      - Initial release (PT);
      20190101:
      - Initial release  - Consolidation into single library for various devices
 * Description:
      This project demonstrates the use of a few MQ Sensors. It enables the user to program a device once, thereafter
      the COM Port is used to change between devices. This provides some degree of convenience when testing and using the
      devices.

      The MQ sesnors are heater-driven sensors that outputs an analog signal. It has fast response and high sensitivity.
      This sensor is suitable for air quality control equipments in buildings/offices.
      This Air Quality detecting Ammonia (NH3), Nitric Oxide and Nitrogen Dioxide (NOx), Alcohol, Benzene, Smoke, CO2, ...
      LCD shows detected concentration in PPM value. The library works with the following sensors: AirQuality click, CO Click, LPG Click, Alcohol Click

      For accurate measuring, gas detector should be considering the temperature and humidity influence.
      For more informations about calibration and accurate measuring please see the sensor datasheet and sensor manufacturer recommendations.

      For accurate reading sensor requires pre heating time of over 24 hours.

     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     ext. modules:    Air Quality click
                      http://www.mikroe.com/click/air-quality/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
      - Place Air Quality Click board in the mikroBUS socket 1.
      - Turn on Lcd backlight switch SW4.6. (board specific)
*/

#include "MQSensor_Functions.h"

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections

const double Rl      = 5000.0;               // Rl (Ohm) - Load resistance
const double Vadc_5  = 0.0048828125;         // ADC step 5V/1024 4,88mV (10bit ADC)
const double Vadc_33 = 0.0032226562;         // ADC step 3,3V/1024 3,22mV (10bit ADC)
double Vrl;                                  // Output voltage
double Rs;                                   // Rs (Ohm) - Sensor resistance
double ppm;                                  // ppm
double ratio;                                // Rs/Rl ratio
char uart_rd;
bit bounce;

unsigned int adc_rd;
unsigned short ADC_Chan,MQSelect;
char txt[16];
char MQ0[] = "AirQuality click";
char MQ1[] = "CO Click";
char MQ2[] = "LPG Click";
char MQ3[] = "Alcohol Click";

//Calculation of PPM
void calculatePPM(unsigned short MQSens) {

  // Input the MQSensor to determine the formula used for calculation
  switch(MQSens){
  
  case 0: ppm = _calcAirQualityPPM(adc_rd, Vadc_5);break;
  case 1: ppm = _calcCOPPM(adc_rd, Vadc_5);break;
  case 2: ppm = _calcLPGPPM(adc_rd, Vadc_5);break;
  case 3: ppm = _calcAlcoholPPM(adc_rd, Vadc_5);break;
  
  }
}

//Read sensor
void readSensor(unsigned short ADC_Chan){

  adc_rd = MQ_Analog(ADC_Chan);              //
  Delay_ms(10);                              // Pause 10ms
}

void LCDDisplay(char Wording[]){

  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,Wording);
  Lcd_Out(2,14,"ppm");                       // Print text on LCD
}

void main() {
  Lcd_Init();                                // Initialize Lcd
  Lcd_Cmd(_LCD_CLEAR);                       // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);                  // Cursor off
  Lcd_Out(1,1,MQ0);                          // Write text in first row

  ANSELA = 0x04;                             // Set ADC on RA2
  TRISA  = 0x04;                             // Set RA2 as input
  ANSELC = 0;
  LATA   = 0;                                // Set PORTA as 0
  ADC_Init();                                // Initialize ADC
  Delay_ms(100);                             // Pause of 100ms for ADC module stabilization
  
  UART1_Init(9600);                          // Initialize UART module at 9600 bps
  Delay_ms(100);                             // Wait for UART module to stabilize

                                             //Initial read ADC and display PPM value on LCD
  ADC_Chan = 2;
  MQSelect = 0;
  Lcd_Out(2,14,"ppm");                       // Print text on LCD
  Lcd_Out(2,1,txt);                          // Print txt value on LCD
  
  while(1) {
  
    Delay_ms(500);                            // Pause 500ms
    readSensor(ADC_Chan);                     // Read sensor
    calculatePPM(MQSelect);                   // Calculating PPM value
    FloatToStr(ppm,&txt);                     // Conversion float ppm to string txt
    Lcd_Out(2,1,"             ");
    Lcd_Out(2,1,txt);                        // Print txt value on LCD
    
    /*
    Uart is triggered as incoming data. Once processed, action taken and bounce 
    flag is only reset at end of routine to avoid unccessary resend if recived data.
    */
    
    if (UART1_Data_Ready()) {               // If data is received,
      
      uart_rd = UART1_Read();               // read the received data,
      UART1_Write(uart_rd);

      switch (uart_rd){
      
      case '0': MQSelect = 0;LCDDisplay(MQ0);break;
      case '1': MQSelect = 1;LCDDisplay(MQ1);break;
      case '2': MQSelect = 2;LCDDisplay(MQ2);break;
      case '3': MQSelect = 3;LCDDisplay(MQ3);break;
      
      }
    }
  }
}